@@ -2,7 +2,7 @@
 DSLIBRARY - run your data science code anywhere.
 
 """
-__version__ = "0.0.6"
+__version__ = "0.0.9"
 
 # environment variables
 from .front import ENV_DSLIBRARY_TARGET, ENV_DSLIBRARY_SPEC, ENV_DSLIBRARY_TOKEN
@@ -68,6 +68,8 @@ def instance(env: dict=None):
     mode, args, kwargs = parse_verb_and_args(target)
     if spec:
         kwargs["spec"] = spec
+    if env is not None:
+        kwargs["_env"] = env
     # a strictly local instance
     if target_parts[0] == "local":
         return DSLibraryLocal(*args, **kwargs)
