@@ -18,7 +18,8 @@ import yaml
 from .proto_handlers import load_protocol_handler
 from .utils.connect import connect_to_database
 from .utils.dbconn import sql_enquote_id
-from .utils.file_utils import write_stream_with_read_on_close, connect_to_filesystem
+from .utils.file_utils import write_stream_with_read_on_close, connect_to_filesystem, find_url_extension
+from .utils.nosql import connect_to_nosql
 
 try:
     import mlflow
@@ -49,7 +50,7 @@ class DSLibrary(object):
     # internal representation of a stored metric
     Metric = namedtuple("Metric", "run_id uri user time name value step")
 
-    def __init__(self, spec: dict=None):
+    def __init__(self, spec: dict=None, _env: dict=None):
         """
         :param spec:    See RunModel for a description of (and a generator for) the specification.  It provides
                         parameter values, and says where input and output data should go, what file format to use, and
@@ -57,6 +58,7 @@ class DSLibrary(object):
                         methods to MLFlow.
         """
         self._spec = spec or {}
+        self._env = _env
         spec_wo_data = dict(self._spec)
         spec_wo_data.pop("data", None)
         jsonschema.validate(spec_wo_data, LOCAL_SPECS_SCHEMA)
@@ -68,7 +70,7 @@ class DSLibrary(object):
             raise DSLibraryException("MLFlow was selected for output but the mlflow package is not installed")
         self._mlflow_run = None
 
-    def _xlt_resource(self, resource_name, _mode="inputs", **kwargs):
+    def _xlt_resource(self, resource_name: str, _mode: str="inputs", **kwargs):
         """
         Look up specifications for a named input or output and use those to override supplied defaults.
         :param resource_name:   Named input.
@@ -193,6 +195,9 @@ class DSLibrary(object):
 
         :param mode:            File read/write/append mode: 'r', 'rb', 'w', 'wb', 'a', 'ab'
 
+        :param kwargs:      Additional arguments:
+                                filesystem = opens the file through a named external filesystem engine
+
         :returns:       A file-like object.
         """
         uri, open_args = self._xlt_resource(resource_name, _mode="inputs" if "r" in mode else "outputs", **kwargs)
@@ -206,9 +211,15 @@ class DSLibrary(object):
                 raise DSLibraryException("Append mode not available with mlflow output")
             return write_stream_with_read_on_close(w_mode=mode, r_mode='r', on_close=lambda fh: mlflow.log_artifact(fh.name, uri))
         # check for a custom protocol handler
-        proto_handler = load_protocol_handler(uri)
+        proto_handler = load_protocol_handler(uri, env=self._env)
         if proto_handler:
             return proto_handler.open_resource(uri, mode, **open_args)
+        # data can be requested through a named filesystem engine/provider
+        filesystem = kwargs.get("filesystem")
+        if filesystem:
+            fs = self.get_filesystem_connection(filesystem, for_write="r" not in mode, **open_args)
+            # TODO some of 'open_args' might be appropriate to send to open() below rather than get_filesystem_connection(), above
+            return fs.open(uri, mode=mode)
         # 'path' has the URI or path of what we need to open
         # 'mode' has the open mode
         # 'open_args' has additional arguments to pass
@@ -253,7 +264,7 @@ class DSLibrary(object):
         """
         # translate 'resource_name' based on supplied specification and overriding supplied arguments
         uri, kwargs = self._xlt_resource(resource_name, "outputs" if for_write else "inputs", **kwargs)
-        handler = load_protocol_handler(uri)
+        handler = load_protocol_handler(uri, env=self._env)
         if handler:
             return handler.get_system_connection(system_type, uri=resource_name, for_write=for_write, **kwargs)
         if system_type == "filesystem":
@@ -261,17 +272,20 @@ class DSLibrary(object):
         if system_type == "sql":
             library = kwargs.pop("library", None)
             return connect_to_database(uri, library=library, for_write=for_write, **kwargs)
-        # TODO implement nosql connection
-        # TODO implement timeseries connection
+        if system_type == "nosql":
+            library = kwargs.pop("library", None)
+            return connect_to_nosql(uri, library=library, for_write=for_write, **kwargs)
         raise ValueError(f"Unsupported system_type: {system_type}")
 
     def get_filesystem_connection(self, resource_name: str, for_write: bool=False, **kwargs):
         """
-        Returns an 'open(filename, mode)' method for a given external filesystem, like s3, abs, gcs, etc..
+        Returns a FileSystem class for a given external filesystem, like s3, abs, gcs, etc..
 
         :param resource_name:   Which filesystem to access.  This can be the name of an input or output resource, or a
                                 URI providing connection details.
         :param for_write:       True to enable write operations.
+        :param kwargs:      Additional arguments, including...
+                bucket = named bucket within the filesystem provider/engine.
         """
         return self._get_system_connection("filesystem", resource_name, for_write, **kwargs)
 
@@ -290,23 +304,12 @@ class DSLibrary(object):
         Returns a connection to a NoSQL database, as an interface that abstracts the functionality of a range of
         popular NoSQL engines.
 
-        :param resource_name:  Which database to access.
-            This can be the name of an input or output resource, or a URI referencing an external data source.
+        :param resource_name:   Which database to access.  This can be the name of an input or output resource, or a
+                                URI referencing an external data source.
+        :param for_write:       True to enable write operations.
         """
         return self._get_system_connection("nosql", resource_name, for_write, **kwargs)
 
-    def get_timeseries_db_connection(self, resource_name: str, for_write: bool=False, **kwargs):
-        """
-        Get a standard interface to a time series database.
-        :param resource_name:   Name of an input or output data connection.  The supplied specifications will map this
-                                to a target database.
-        :param for_write:       True to enable write access.
-        :param kwargs:      Additional connection configuration, credentials, etc.
-        :return:    A connection, as a yet-to-be-determined interface that supports a range of popular time series
-                    databases.
-        """
-        return self._get_system_connection("timeseries", resource_name, for_write, **kwargs)
-
     def log_param(self, key: str, value):
         """
         Log a supplied parameter.
@@ -517,8 +520,7 @@ class DSLibrary(object):
         use_mlflow = self._mlflow_all
         # map resource name using supplied specs
         uri, kwargs = self._xlt_resource(resource_name, _mode="outputs", **kwargs)
-        # FIXME use the URL extension splitter!
-        file_extension = os.path.splitext(uri)[1].strip(".")
+        file_extension = find_url_extension(uri).strip(".")
         if not file_extension:
             self._default_format_for_aliases(resource_name, kwargs)
         format = kwargs.pop("format", None) or file_extension
@@ -532,7 +534,6 @@ class DSLibrary(object):
         if format_options.get("nosql_collection"):
             # TODO detect nosql (nosql_collection) and write data there
             raise DSLibraryException("NoSQL write of columnar data is not yet implemented")
-        # TODO support for time series databases
         # we detect numpy arrays, series, etc., and save them as a dataframe with one column
         if isinstance(content, (pandas.Series, numpy.ndarray, pandas.Index)):
             series = content
@@ -706,6 +707,24 @@ class DSLibrary(object):
         else:
             raise ValueError("expected content or filename")
 
+    def multi_part_upload(self, resource_name: str, chunk: (bytes, bytearray), upload_id: str=None, sequence: int=None, done: bool=False, **kwargs):
+        """
+        Manage a large upload.
+
+        This is a placeholder for proper upload functionality for very large files, particularly to systems like S3 that
+        do not support append.
+
+        :param chunk:       Data to send.
+        :param upload_id:   Unique identifier for the upload, returned from the first call.
+        :param sequence:    Chunks can be delivered out of sequence, i.e. by separate processes.
+        :param done:        Set this flag on the last chunk.
+        :return:        An upload identifier.
+        """
+        # TODO this placeholder implementation just appends to a file.  This won't work for systems like s3.
+        with self.open_resource(resource_name, mode='ab' if upload_id else 'wb', **kwargs) as f_w:
+            f_w.write(chunk)
+        return "-"
+
     def load_dataframe(self, resource_name: str, **kwargs):
         """
         Load a pandas dataframe from various types of input resources.
@@ -714,12 +733,12 @@ class DSLibrary(object):
             uri - path or URI to target
             format - csv, json, etc
             format_options - these are passed to pandas.read_csv(), etc.
+            filesystem - name of a filesystem
             sql - SQL to execute
             sql_table - name of table to fully load
             nosql_collection - name of a NoSQL collection to fully load
 
         :param resource_name:   The name of an input resource, or of a local file, or the URI of an external resource.
-                For sql- and nosql- type data sources, ":<table_name>" is expected.
         """
         # check for specifically supplied data
         supplied = (self._spec.get("data") or {}).get(resource_name)
@@ -733,7 +752,7 @@ class DSLibrary(object):
             table = open_args.pop("sql_table", None)
             if table:
                 sql = f"SELECT * from {sql_enquote_id(table)}"
-            db_conn = self.get_sql_connection(resource_name)
+            db_conn = self.get_sql_connection(resource_name, **open_args)
             try:
                 c = db_conn.cursor()
                 c.execute(sql)
@@ -742,15 +761,15 @@ class DSLibrary(object):
             finally:
                 db_conn.close()
             return df
+        # detect NoSQL
         if open_args.get("nosql_collection"):
-            collection = open_args["nosql_collection"]
-            # TODO code get_nosql_connection() then implement accordingly
-            raise DSLibraryException("Loading from nosql is not supported yet.")
-        # TODO support read from time series database
+            collection = open_args.pop("nosql_collection")
+            db_conn = self.get_nosql_connection(resource_name, **open_args)
+            rows = list(db_conn.query(collection))
+            return pandas.DataFrame(rows)
         # TODO we should offer a validation option that checks the data types against the schema
         # format can be specified, or file extension will be used
-        # FIXME use the URL splitter, not this one!
-        file_extension = os.path.splitext(uri)[1].strip(".")
+        file_extension = find_url_extension(uri).strip(".")
         if not file_extension:
             self._default_format_for_aliases(resource_name, open_args)
         # options for read function can be given
