@@ -7,6 +7,7 @@ import io
 import json
 import inspect
 
+from dslibrary.engine_intf import FileSystem
 from dslibrary.front import DSLibraryException
 from dslibrary.metadata import Metadata
 from dslibrary.utils.file_utils import write_stream_with_read_on_close
@@ -75,6 +76,25 @@ class DSLibraryViaMMLibrary(DSLibrary):
                 raise FileNotFoundError(f"not found: {path}")
         raise DSLibraryException(f"Unsupported mode: {mode}")
 
+    def get_filesystem_connection(self, resource_name: str, for_write: bool=False, **kwargs):
+        open_method = self._find_method("open_stream")
+        if not open_method:
+            raise DSLibraryException(f"No filesystem engine access available, requested={resource_name}")
+        bucket = kwargs.get("bucket")
+        file_source = f"{resource_name}:{bucket}" if bucket else resource_name
+        class MMFS(FileSystem):
+            def ls(self, path, detail=True, **kwargs):
+                raise NotImplementedError()
+            def rm(self, path, recursive=False, maxdepth=None):
+                raise NotImplementedError()
+            def mkdir(self, path, create_parents=True, **kwargs):
+                raise NotImplementedError()
+            def stat(self, path, **kwargs):
+                raise NotImplementedError()
+            def open(self, path, mode="rb", block_size=None, cache_options=None, **kwargs):
+                return open_method(path, file_source=file_source, mode=mode)
+        return MMFS()
+
     def open_model_binary(self, part: str=None, mode: str='rb') -> io.RawIOBase:
         """
         The model-binary is just one file, but we can support 'part' by assuming it is a ZIP file.
@@ -134,8 +154,8 @@ class DSLibraryViaMMLibrary(DSLibrary):
     def get_next_scoring_request(self, timeout: float=None) -> (dict, None):
         raise DSLibraryException("not implemented in mmlibrary, use get_parameter() for each fields of a single scoring request")
 
-    def send_score(self, score):
-        method = self._find_method("report_score", "reportScore")
+    def scoring_response(self, score):
+        method = self._find_method("return_score", "returnScore")
         method(score)
 
     def get_sql_connection(self, resource_name: str, for_write: bool=False, database: str=None, **kwargs):
