@@ -7,13 +7,12 @@ import re
 import io
 import time
 
-from ..metadata import Metadata
-
 try:
     import requests
 except ImportError:
     requests = None
 
+from ..metadata import Metadata
 from ..front import DSLibrary, DSLibraryCommunicationError, DSLibraryException
 from ..utils import filechunker
 from ..utils import dbconn
@@ -127,18 +126,17 @@ class DSLibraryViaREST(DSLibrary):
         """
         self._do_comm("post", "score", params={"value": score})
 
-    def get_sql_connection(self, resource_name: str, for_write: bool=False, database: str=None, **kwargs):
+    def get_sql_connection(self, resource_name: str, for_write: bool=False, **kwargs):
         """
         SQL conversation is 'chunked across REST', so to speak.
         """
         if not isinstance(resource_name, str):
             raise ValueError(f"get_db_connection(): expected str for 'connection_name', got {type(resource_name)}")
-        if database:
-            connection_name = "%s:%s" % (resource_name, database)
-        elif ":" in resource_name:
-            database = resource_name.split(":")[-1]
+        params = dict(resource_name=resource_name)
 
         def read(operation, parameters):
+            # FIXME re-establish this upstream
+            '''
             # pandas makes this specific query to verify a table does not already exist (pandas.DataFrame.to_sql())
             if "select name from sqlite_master" in operation.lower():
                 # convert to standard SQL
@@ -146,9 +144,9 @@ class DSLibraryViaREST(DSLibrary):
                 # try to qualify based on database
                 if database:
                     operation += f" and table_schema='{database}'"
+            '''
             resp = self._do_comm(
-                "get", "db/%s" % connection_name,
-                params=dict(sql=operation, parameters=json.dumps(parameters))
+                "get", "db", params=dict(**params, sql=operation, parameters=json.dumps(parameters))
             )
             cols = resp[0]
             rows = resp[1:]
@@ -161,7 +159,7 @@ class DSLibraryViaREST(DSLibrary):
         def read_more(chunk):
             resp = None
             for _ in range(15):
-                resp = self._do_comm("get", "db_more/%s" % chunk, params=dict(max_wait=60))
+                resp = self._do_comm("get", "db", params=dict(**params, more=chunk, max_wait=60))
                 if resp:
                     break
             if not resp:
@@ -173,6 +171,6 @@ class DSLibraryViaREST(DSLibrary):
             return rows, more
 
         def write(operation, parameters):
-            self._do_comm("post", "db/%s" % connection_name, data=dict(sql=operation, parameters=parameters))
+            self._do_comm("post", "db", data=dict(**params, sql=operation, parameters=parameters))
 
-        return dbconn.Connection(read, write, read_more=read_more)
+        return dbconn.Connection(read, write if for_write else None, read_more=read_more)
