@@ -7,7 +7,7 @@ import tempfile
 import typing
 import urllib.request
 
-from dslibrary.utils.filechunker import ChunkedFileWriter
+from dslibrary.engine_intf import FileSystemLocal, FileSystemReadOnly
 
 try:
     import fsspec
@@ -119,23 +119,34 @@ def write_stream_with_read_on_close(w_mode: str, r_mode: str, on_close: typing.C
 def connect_to_filesystem(uri: str, for_write: bool=False, **kwargs):
     """
     Generate an open(filename, mode) method based on a URI and credentials.
-    :param uri:             A partial URI like "s3://bucket".
+    :param uri:             A partial URI like "s3://bucket", or a local path.
     :param for_write:       Whether to allow write access.
     :param kwargs:          Additional arguments, namely credentials.
-    :return:        An open() method that lets you access files in the remote filesystem, for read or write.
+    :return:            A FileSystem instance that lets you work with files in the remote filesystem.
     """
-    # uri can just be a local path
     if "://" not in uri:
-        uri = os.path.abspath(uri)
-        return lambda filename, mode: open(os.path.join(uri, filename.strip("/")), mode)
-    protocol, bucket = uri.strip("/").split("://", maxsplit=1)
-    if "/" in bucket or ":" in bucket:
-        raise ValueError("Invalid URI for external filesystem")
+        # uri can just be a local path
+        fs = FileSystemLocal(uri)
+    else:
+        # otherwise we assume the usual cloud format: protocol://bucket
+        protocol, bucket = uri.strip("/").split("://", maxsplit=1)
+        if "/" in bucket or ":" in bucket:
+            raise ValueError(f"Invalid URI for external filesystem: {uri}")
+        if not fsspec:
+            raise ImportWarning(f"fsspec not installed, can't open filesystem: {uri}")
+        fs = fsspec.filesystem(protocol, bucket=bucket, **kwargs)
+    if not for_write:
+        return FileSystemReadOnly(fs)
+    return fs
 
-    def opener(filename, mode):
-        if "w" in mode and not for_write:
-            raise PermissionError("write not allowed on this filesystem")
-        uri = f"{protocol}://{bucket}/{filename.strip('/')}"
-        return FileOpener().open(uri, mode, **kwargs)
 
-    return opener
+def find_url_extension(url: str) -> str:
+    """
+    Find the extension of a filename or URL.
+    """
+    url = url or ""
+    if "#" in url:
+        url = url.split("#")[0]
+    if "?" in url:
+        url = url.split("?")[0]
+    return os.path.splitext(url)[1].lower().strip(".")
