@@ -2,7 +2,7 @@ import os
 import io
 import unittest
 
-from dslibrary import DSLibrary
+from dslibrary import DSLibrary, instance
 from dslibrary.proto_handlers import load_protocol_handler, ProtocolHandler
 
 
@@ -19,13 +19,19 @@ class TestProtoHandlers(unittest.TestCase):
 
     def test_through_open_resource(self):
         os.environ["DSLIBRARY_PROTO_ZZZ"] = "tests.test_proto_handlers.MyProtoZZZ:v=hello2"
-        handler = load_protocol_handler("zzz://host/etc")
         with DSLibrary().open_resource("x", "rb", uri="zzz://x") as f:
             assert f.read() == b'hello2'
 
     def test_through_get_filesystem(self):
-        # TODO access a filesystem over the 'zzz' protocol
-        pass
+        os.environ["DSLIBRARY_PROTO_ZZZ"] = "tests.test_proto_handlers.MyProtoZZZ:v=hello2"
+        fs = DSLibrary().get_filesystem_connection("zzz://x")
+        with fs.open("x") as f_r:
+            assert f_r.read() == b'from_fs'
+
+    def test_through_dslibrary_instance__with_custom_env(self):
+        dsl = instance({"DSLIBRARY_PROTO_ZZZ": "tests.test_proto_handlers.MyProtoZZZ:v=hello3"})
+        r = dsl.read_resource("zzz://x")
+        assert r == b'hello3'
 
 
 class MyProtoZZZ(ProtocolHandler):
@@ -34,3 +40,9 @@ class MyProtoZZZ(ProtocolHandler):
 
     def open_resource(self, url: str, mode: str, **kwargs):
         return io.BytesIO(self.v.encode("utf-8"))
+
+    def get_system_connection(self, system_type, uri: str, for_write: bool=False, **kwargs):
+        class MyFS(object):
+            def open(self, name):
+                return io.BytesIO(b'from_fs')
+        return MyFS()
\ No newline at end of file
